use openssl::{
    bn::{BigNum, BigNumContext},
    ec::{EcGroup, EcPoint},
};

pub fn initialize_elliptic_curve(
    p_sel: &str,
    a_sel: &str,
    b_sel: &str,
    generator_x_sel: &str,
    generator_y_sel: &str,
    public_key_x_sel: &str,
    public_key_y_sel: &str,
) -> (EcGroup, EcPoint, EcPoint) {
    let mut context = BigNumContext::new().unwrap();

    let p = BigNum::from_dec_str(p_sel).unwrap();
    let a = BigNum::from_dec_str(a_sel).unwrap();
    let b = BigNum::from_dec_str(b_sel).unwrap();
    let generator_x = BigNum::from_dec_str(generator_x_sel).unwrap();
    let generator_y = BigNum::from_dec_str(generator_y_sel).unwrap();

    let public_key_x = BigNum::from_dec_str(public_key_x_sel).unwrap();
    let public_key_y = BigNum::from_dec_str(public_key_y_sel).unwrap();

    let c_curve = EcGroup::from_components(p, a, b, &mut context).unwrap();

    let mut gen_point = EcPoint::new(&c_curve).unwrap();
    let _ =
        gen_point.set_affine_coordinates_gfp(&c_curve, &generator_x, &generator_y, &mut context);

    let mut pub_point = EcPoint::new(&c_curve).unwrap();
    let _ =
        pub_point.set_affine_coordinates_gfp(&c_curve, &public_key_x, &public_key_y, &mut context);

    (c_curve, gen_point, pub_point)
}
